@@ -502,7 +502,7 @@ static void handle_flags(struct sr_datafeed_analog *analog,
 	if (info->is_auto)
 		analog->meaning->mqflags |= SR_MQFLAG_AUTORANGE;
 	if (info->is_diode)
-		analog->meaning->mqflags |= SR_MQFLAG_DIODE;
+		analog->meaning->mqflags |= SR_MQFLAG_DIODE | SR_MQFLAG_DC;
 	if (info->is_hold)
 		/*
 		* Note: HOLD only affects the number displayed on the LCD,
