@@ -329,7 +329,7 @@ static gboolean flags_valid(const struct metex14_info *info)
 	return TRUE;
 }
 
-#ifdef HAVE_LIBSERIALPORT
+#ifdef HAVE_SERIAL_COMM
 SR_PRIV int sr_metex14_packet_request(struct sr_serial_dev_inst *serial)
 {
 	const uint8_t wbuf = 'D';
