@@ -346,7 +346,8 @@ static void process_byte(const struct sr_dev_inst *sdi, const unsigned char c,
 					g_variant_new_uint64(devc->buf[7] * 1000));
 			meta.config = g_slist_append(NULL, src);
 			sr_session_send(sdi, &packet);
-			g_free(src);
+			g_slist_free(meta.config);
+			sr_config_free(src);
 			devc->buf_len = 0;
 		}
 	} else if (devc->state == ST_GET_LOG_RECORD_DATA) {
