@@ -18,13 +18,12 @@
  */
 
 #include <config.h>
-#include <libserialport.h>
 #include <scpi.h>
 #include <string.h>
 
 #include "protocol.h"
 
-SR_PRIV struct sr_dev_driver rohde_schwarz_sme_0x_driver_info;
+static struct sr_dev_driver rohde_schwarz_sme_0x_driver_info;
 
 static const char *manufacturer = "Rohde&Schwarz";
 
@@ -220,7 +219,7 @@ static int config_list(uint32_t key, GVariant **data,
 	return STD_CONFIG_LIST(key, data, sdi, cg, scanopts, drvopts, devopts);
 }
 
-SR_PRIV struct sr_dev_driver rohde_schwarz_sme_0x_driver_info = {
+static struct sr_dev_driver rohde_schwarz_sme_0x_driver_info = {
 	.name = "rohde-schwarz-sme-0x",
 	.longname = "Rohde&Schwarz SME-0x",
 	.api_version = 1,
@@ -238,5 +237,4 @@ SR_PRIV struct sr_dev_driver rohde_schwarz_sme_0x_driver_info = {
 	.dev_acquisition_stop = std_serial_dev_acquisition_stop,
 	.context = NULL,
 };
-
 SR_REGISTER_DEV_DRIVER(rohde_schwarz_sme_0x_driver_info);
