@@ -201,35 +201,35 @@ static const struct siglent_sds_series supported_series[] = {
 #define SERIES(x) &supported_series[x]
 /* series, model, min timebase, analog channels, digital */
 static const struct siglent_sds_model supported_models[] = {
-	{ SERIES(SDS1000CML), "SDS1152CML", { 20, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000CML), "SDS1102CML", { 10, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000CML), "SDS1072CML", { 5, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000CNL), "SDS1202CNL", { 20, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000CNL), "SDS1102CNL", { 10, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000CNL), "SDS1072CNL", { 5, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000DL), "SDS1202DL", { 20, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000DL), "SDS1102DL", { 10, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000DL), "SDS1022DL", { 5, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000DL), "SDS1052DL", { 5, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000DL), "SDS1052DL+", { 5, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000X), "SDS1102X", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000XP), "SDS1102X+", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000X), "SDS1202X", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000XP), "SDS1202X+", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000XE), "SDS1202X-E", { 1, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS1000XE), "SDS1104X-E", { 1, 1000000000 }, 4, true, 16 },
-	{ SERIES(SDS1000XE), "SDS1204X-E", { 1, 1000000000 }, 4, true, 16 },
-	{ SERIES(SDS2000X), "SDS2072X", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS2000X), "SDS2074X", { 2, 1000000000 }, 4, false, 0 },
-	{ SERIES(SDS2000X), "SDS2102X", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS2000X), "SDS2104X", { 2, 1000000000 }, 4, false, 0 },
-	{ SERIES(SDS2000X), "SDS2202X", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS2000X), "SDS2204X", { 2, 1000000000 }, 4, false, 0 },
-	{ SERIES(SDS2000X), "SDS2302X", { 2, 1000000000 }, 2, false, 0 },
-	{ SERIES(SDS2000X), "SDS2304X", { 2, 1000000000 }, 4, false, 0 },
+	{ SERIES(SDS1000CML), "SDS1152CML", { 20, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000CML), "SDS1102CML", { 10, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000CML), "SDS1072CML", { 5, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000CNL), "SDS1202CNL", { 20, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000CNL), "SDS1102CNL", { 10, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000CNL), "SDS1072CNL", { 5, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000DL), "SDS1202DL", { 20, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000DL), "SDS1102DL", { 10, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000DL), "SDS1022DL", { 5, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000DL), "SDS1052DL", { 5, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000DL), "SDS1052DL+", { 5, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000X), "SDS1102X", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000XP), "SDS1102X+", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000X), "SDS1202X", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000XP), "SDS1202X+", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000XE), "SDS1202X-E", { 1, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS1000XE), "SDS1104X-E", { 1, 1000000000 }, 4, TRUE, 16 },
+	{ SERIES(SDS1000XE), "SDS1204X-E", { 1, 1000000000 }, 4, TRUE, 16 },
+	{ SERIES(SDS2000X), "SDS2072X", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS2000X), "SDS2074X", { 2, 1000000000 }, 4, FALSE, 0 },
+	{ SERIES(SDS2000X), "SDS2102X", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS2000X), "SDS2104X", { 2, 1000000000 }, 4, FALSE, 0 },
+	{ SERIES(SDS2000X), "SDS2202X", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS2000X), "SDS2204X", { 2, 1000000000 }, 4, FALSE, 0 },
+	{ SERIES(SDS2000X), "SDS2302X", { 2, 1000000000 }, 2, FALSE, 0 },
+	{ SERIES(SDS2000X), "SDS2304X", { 2, 1000000000 }, 4, FALSE, 0 },
 };
 
-SR_PRIV struct sr_dev_driver siglent_sds_driver_info;
+static struct sr_dev_driver siglent_sds_driver_info;
 
 static void clear_helper(void *priv)
 {
@@ -923,7 +923,7 @@ static int dev_acquisition_stop(struct sr_dev_inst *sdi)
 	return SR_OK;
 }
 
-SR_PRIV struct sr_dev_driver siglent_sds_driver_info = {
+static struct sr_dev_driver siglent_sds_driver_info = {
 	.name = "siglent-sds",
 	.longname = "Siglent SDS1000/SDS2000",
 	.api_version = 1,
@@ -941,5 +941,4 @@ SR_PRIV struct sr_dev_driver siglent_sds_driver_info = {
 	.dev_acquisition_stop = dev_acquisition_stop,
 	.context = NULL,
 };
-
 SR_REGISTER_DEV_DRIVER(siglent_sds_driver_info);
