@@ -183,6 +183,22 @@ static struct sr_key_info sr_key_info_config[] = {
 		"Trigger level", NULL},
 	{SR_CONF_EXTERNAL_CLOCK_SOURCE, SR_T_STRING, "external_clock_source",
 		"External clock source", NULL},
+	{SR_CONF_OFFSET, SR_T_FLOAT, "offset",
+		"Offset", NULL},
+	{SR_CONF_TRIGGER_PATTERN, SR_T_STRING, "triggerpattern",
+		"Trigger pattern", NULL},
+	{SR_CONF_HIGH_RESOLUTION, SR_T_BOOL, "highresolution",
+		"High resolution", NULL},
+	{SR_CONF_PEAK_DETECTION, SR_T_BOOL, "peakdetection",
+		"Peak detection", NULL},
+	{SR_CONF_LOGIC_THRESHOLD, SR_T_STRING, "logic_threshold",
+		"Logic threshold (predefined)", NULL},
+	{SR_CONF_LOGIC_THRESHOLD_CUSTOM, SR_T_FLOAT, "logic_threshold_custom",
+		"Logic threshold (custom)", NULL},
+	{SR_CONF_RANGE, SR_T_STRING, "range",
+		"Range", NULL},
+	{SR_CONF_DIGITS, SR_T_STRING, "digits",
+		"Digits", NULL},
 
 	/* Special stuff */
 	{SR_CONF_SESSIONFILE, SR_T_STRING, "sessionfile",
@@ -569,6 +585,12 @@ SR_PRIV void sr_hw_cleanup_all(const struct sr_context *ctx)
  *
  * A floating reference can be passed in for data.
  *
+ * @param key The config key to use.
+ * @param data The GVariant data to use.
+ *
+ * @return The newly allocated struct sr_config. This function is assumed
+ *         to never fail.
+ *
  * @private
  */
 SR_PRIV struct sr_config *sr_config_new(uint32_t key, GVariant *data)
