@@ -44,7 +44,7 @@
  *          Values are "channel", "units", "off". Defaults to "units".
  *
  * time:    Whether or not the first column should include the time the sample
- *          was taken. Defaults to TRUE.
+ *          was taken. Defaults to FALSE.
  *
  * trigger: Whether or not to add a "trigger" column as the last column.
  *          Defaults to FALSE.
@@ -252,7 +252,7 @@ static GString *gen_header(const struct sr_output *o,
 		g_string_append_printf(header,
 			"%s CSV generated by %s %s\n%s from %s on %s",
 			ctx->comment, PACKAGE_NAME,
-			SR_PACKAGE_VERSION_STRING, ctx->comment,
+			sr_package_version_string_get(), ctx->comment,
 			ctx->title, ctime(&hdr->starttime.tv_sec));
 
 		/* Columns / channels */
@@ -658,7 +658,7 @@ static const struct sr_option *get_options(void)
 		l = g_slist_append(l, g_variant_ref_sink(g_variant_new_string("channel")));
 		l = g_slist_append(l, g_variant_ref_sink(g_variant_new_string("off")));
 		options[7].values = l;
-		options[8].def = g_variant_ref_sink(g_variant_new_boolean(TRUE));
+		options[8].def = g_variant_ref_sink(g_variant_new_boolean(FALSE));
 		options[9].def = g_variant_ref_sink(g_variant_new_boolean(FALSE));
 		options[10].def = g_variant_ref_sink(g_variant_new_boolean(FALSE));
 	}
